@@ -63,7 +63,8 @@ public:
     void cleanup();
 
     /// @brief Initiates shutdown procedure for the whole DHCPv6 server.
-    void shutdown();
+    /// @param exit_value integer value to the process should exit with.
+    virtual void shutdownServer(int exit_value);
 
     /// @brief Command processor
     ///
