@@ -14,6 +14,7 @@
 #include <dhcp/option.h>
 #include <cfgrpt/config_report.h>
 #include <util/encode/hex.h>
+#include <asiolink/io_error.h>
 
 #include <boost/lexical_cast.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
@@ -116,6 +117,8 @@ CommandOptions::reset() {
     renew_rate_ = 0;
     release_rate_ = 0;
     report_delay_ = 0;
+    clean_report_ = false;
+    clean_report_separator_ = "";
     clients_num_ = 0;
     mac_template_.assign(mac, mac + 6);
     duid_template_.clear();
@@ -123,6 +126,9 @@ CommandOptions::reset() {
     addr_unique_ = false;
     mac_list_file_.clear();
     mac_list_.clear();
+    giaddr_list_file_.clear();
+    giaddr_list_.clear();
+    multi_subnet_ = false;
     num_request_.clear();
     exit_wait_time_ = 0;
     period_ = 0;
@@ -223,6 +229,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
     std::ostringstream stream;
     stream << "perfdhcp";
     int num_mac_list_files = 0;
+    int num_subnet_list_files = 0;
 
     struct option long_options[] = {
         {"scenario", required_argument, 0, LONG_OPT_SCENARIO},
@@ -233,7 +240,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
     // they will be tuned and validated elsewhere
     while((opt = getopt_long(argc, argv,
                              "huv46A:r:t:R:b:n:p:d:D:l:P:a:L:N:M:s:iBc1"
-                             "T:X:O:o:E:S:I:x:W:w:e:f:F:g:",
+                             "J:T:X:O:o:E:S:I:x:W:w:e:f:F:g:C:",
                              long_options, NULL)) != -1) {
         stream << " -" << static_cast<char>(opt);
         if (optarg) {
@@ -286,6 +293,11 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             rapid_commit_ = true;
             break;
 
+        case 'C':
+            clean_report_ = true;
+            clean_report_separator_ = optarg;
+            break;
+
         case 'd':
             check(drop_time_set_ > 1,
                   "maximum number of drops already specified, "
@@ -373,6 +385,13 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
                                           " positive integer");
             break;
 
+        case 'J':
+            check(num_subnet_list_files >= 1, "only one -J option can be specified");
+            num_subnet_list_files++;
+            giaddr_list_file_ = std::string(optarg);
+            loadGiaddr();
+            break;
+
         case 'l':
             localname_ = std::string(optarg);
             initIsInterface();
@@ -399,7 +418,7 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             break;
 
         case 'M':
-            check(num_mac_list_files >= 1, "only -M option can be specified");
+            check(num_mac_list_files >= 1, "only one -M option can be specified");
             num_mac_list_files++;
             mac_list_file_ = std::string(optarg);
             loadMacs();
@@ -610,19 +629,20 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             server_name_ = ALL_DHCP_SERVERS;
         }
     }
+    if (!getCleanReport()) {
+        if (print_cmd_line) {
+            std::cout << "Running: " << stream.str() << std::endl;
+        }
 
-    if (print_cmd_line) {
-        std::cout << "Running: " << stream.str() << std::endl;
-    }
+        if (scenario_ == Scenario::BASIC) {
+            std::cout << "Scenario: basic." << std::endl;
+        } else if (scenario_ == Scenario::AVALANCHE) {
+            std::cout << "Scenario: avalanche." << std::endl;
+        }
 
-    if (scenario_ == Scenario::BASIC) {
-        std::cout << "Scenario: basic." << std::endl;
-    } else if (scenario_ == Scenario::AVALANCHE) {
-        std::cout << "Scenario: avalanche." << std::endl;
-    }
-
-    if (!isSingleThreaded()) {
-        std::cout << "Multi-thread mode enabled." << std::endl;
+        if (!isSingleThreaded()) {
+            std::cout << "Multi-thread mode enabled." << std::endl;
+        }
     }
 
     // Handle the local '-l' address/interface
@@ -816,6 +836,31 @@ CommandOptions::convertHexString(const std::string& text) const {
     return ui;
 }
 
+bool CommandOptions::validateIP(const std::string& line) {
+    try {
+        asiolink::IOAddress ip_address_ = isc::asiolink::IOAddress(line);
+        // let's silence not used warning
+        (void) ip_address_;
+    } catch (const isc::asiolink::IOError& e) {
+        return (true);
+    }
+    giaddr_list_.push_back(line);
+    multi_subnet_ = true;
+    return (false);
+}
+
+void CommandOptions::loadGiaddr() {
+    std::string line;
+    std::ifstream infile(giaddr_list_file_.c_str());
+    size_t cnt = 0;
+    while (std::getline(infile, line)) {
+        cnt++;
+        stringstream tmp;
+        tmp << "invalid address in line: "<< cnt;
+        check(validateIP(line), tmp.str());
+    }
+}
+
 void CommandOptions::loadMacs() {
     std::string line;
     std::ifstream infile(mac_list_file_.c_str());
@@ -1107,8 +1152,8 @@ CommandOptions::usage() const {
     std::cout <<
         "perfdhcp [-huv] [-4|-6] [-A<encapsulation-level>] [-e<lease-type>]\n"
         "         [-r<rate>] [-f<renew-rate>]\n"
-        "         [-F<release-rate>] [-t<report>] [-R<range>] [-b<base>]\n"
-        "         [-n<num-request>] [-p<test-period>] [-d<drop-time>]\n"
+        "         [-F<release-rate>] [-t<report>] [-C<separator>] [-R<range>]\n"
+        "         [-b<base>] [-n<num-request>] [-p<test-period>] [-d<drop-time>]\n"
         "         [-D<max-drop>] [-l<local-addr|interface>] [-P<preload>]\n"
         "         [-L<local-port>] [-N<remote-port>]\n"
         "         [-s<seed>] [-i] [-B] [-W<late-exit-delay>]\n"
@@ -1177,6 +1222,9 @@ CommandOptions::usage() const {
         "    whether -6 is given.\n"
         "-I<ip-offset>: Offset of the (DHCPv4) IP address in the requested-IP\n"
         "    option / (DHCPv6) IA_NA option in the (second/request) template.\n"
+        "-J<giaddr-list-file>: Text file that include multiple addresses.\n"
+        "    If provided perfdhcp will choose randomly one of addresses for each\n"
+        "    exchange.\n"
         "-l<local-addr|interface>: For DHCPv4 operation, specify the local\n"
         "    hostname/address to use when communicating with the server.  By\n"
         "    default, the interface address through which traffic would\n"
@@ -1258,6 +1306,9 @@ CommandOptions::usage() const {
         "    alternative to -n, or both options can be given, in which case the\n"
         "    testing is completed when either limit is reached.\n"
         "-t<report>: Delay in seconds between two periodic reports.\n"
+        "-C<separator>: Output reduced, an argument is a separator for periodic\n"
+        "    (-t) reports generated in easy parsable mode. Data output won't be\n"
+        "    changed, remain identical as in -t option.\n"
         "\n"
         "Errors:\n"
         "- tooshort: received a too short message\n"
