@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -26,7 +26,7 @@ Pool::Pool(Lease::Type type, const isc::asiolink::IOAddress& first,
 }
 
 bool Pool::inRange(const isc::asiolink::IOAddress& addr) const {
-    return (first_.smallerEqual(addr) && addr.smallerEqual(last_));
+    return (first_ <= addr && addr <= last_);
 }
 
 bool Pool::clientSupported(const ClientClasses& classes) const {
